Jinja2 Documentation

# Jinja

« Introduction | API | Sandbox »

# API

This document describes the API to Jinja2 and not the template language. It will be most useful as reference to those implementing the template interface to the application and not those who are creating Jinja2 templates.

## Basics

Jinja2 uses a central object called the template `Environment`. Instances of this class are used to store the configuration, global objects and are used to load templates from the file system or other locations. Even if you are creating templates from string by using the constructor of `Template` class, an environment is created automatically for you, albeit a shared one.

Most applications will create one `Environment` object on application initialization and use that to load templates. In some cases it’s however useful to have multiple environments side by side, if different configurations are in use.

The simplest way to configure Jinja2 to load templates for your application looks roughly like this:

    from jinja2 import Environment, PackageLoader
    env = Environment(loader=PackageLoader('yourapplication', 'templates'))

This will create a template environment with the default settings and a loader that looks up the templates in the templates folder inside the yourapplication python package. Different loaders are available and you can also write your own if you want to load templates from a database or other resources.

To load a template from this environment you just have to call the `get_template()` method which then returns the loaded `Template`:

    template = env.get_template('mytemplate.html')

To render it with some variables, just call the `render()` method:

    print template.render(the='variables', go='here')

Using a template loader rather then passing strings to `Template` or `Environment.from_string()` has multiple advantages. Besides being a lot easier to use it also enables template inheritance.

## Unicode

Jinja2 is using unicode internally which means that you have to pass unicode objects to the render function or bytestrings that only consist of ASCII characters. Additionally newlines are normalized to one end of line sequence which is per default UNIX style (`\n`).

Python 2.x supports two ways of representing string objects. One is the str type and the other is the unicode type, both of which extend a type called basestring. Unfortunately the default is str which should not be used to store text based information unless only ASCII characters are used. With Python 2.6 it is possible to my unicode the default on a per module level and with Python 3 it will be the default.

To explicitly use a unicode string you have to prefix the string literal with a u: `u'Hänsel und Gretel sagen Hallo'`. That way Python will store the string as unicode by decoding the string with the character encoding from the current Python module. If no encoding is specified this defaults to ‘ASCII’ which means that you can’t use any non ASCII identifier.

To set a better module encoding add the following comment to the first or second line of the Python module using the unicode literal:

    # -*- coding: utf-8 -*-

We recommend utf-8 as Encoding for Python modules and templates as it’s possible to represent every Unicode character in utf-8 and because it’s backwards compatible to ASCII. For Jinja2 the default encoding of templates is assumed to be utf-8.

It is not possible to use Jinja2 to process non unicode data. The reason for this is that Jinja2 uses Unicode already on the language level. For example Jinja2 treats the non-breaking space as valid whitespace inside expressions which requires knowledge of the encoding or operating on an unicode string.

For more details about unicode in Python have a look at the excellent Unicode documentation.

Another important thing is how Jinja2 is handling string literals in templates. A naive implementation would be using unicode strings for all string literals but it turned out in the past that this is problematic as some libraries are typechecking against str explicitly. For example datetime.strftime does not accept unicode arguments. To not break it completely Jinja2 is returning str for strings that fit into ASCII and for everything else unicode:

    >>> m = Template(u"{% set a, b = 'foo', 'föö' %}").module
    >>> m.a
    'foo'
    >>> m.b
    u'f\xf6\xf6'

## High Level API

The high-level API is the API you will use in the application to load and render Jinja2 templates. The Low Level API on the other side is only useful if you want to dig deeper into Jinja2 or develop extensions.

class `jinja2.` `Environment`([options])

  The core component of Jinja is the Environment. It contains important shared variables like configuration, filters, tests, globals and others. Instances of this class may be modified if they are not shared and if no template was loaded so far. Modifications on environments after the first template was loaded will lead to surprising effects and undefined behavior.

  Here the possible initialization parameters:

  > block_start_string
  >   The string marking the begin of a block. Defaults to `'{%'`.
  > block_end_string
  >   The string marking the end of a block. Defaults to `'%}'`.
  > variable_start_string
  >   The string marking the begin of a print statement. Defaults to `'{{'`.
  > variable_end_string
  >   The string marking the end of a print statement. Defaults to `'}}'`.
  > comment_start_string
  >   The string marking the begin of a comment. Defaults to `'{#'`.
  > comment_end_string
  >   The string marking the end of a comment. Defaults to `'#}'`.
  > line_statement_prefix
  >   If given and a string, this will be used as prefix for line based statements. See also Line Statements.
  > trim_blocks
  >   If this is set to `True` the first newline after a block is removed (block, not variable tag!). Defaults to False.
  > newline_sequence
  >   The sequence that starts a newline. Must be one of `'\r'`, `'\n'` or `'\r\n'`. The default is `'\n'` which is a useful default for Linux and OS X systems as well as web applications.
  > extensions
  >   List of Jinja extensions to use. This can either be import paths as strings or extension classes. For more information have a look at the extensions documentation.
  > optimized
  >   should the optimizer be enabled? Default is True.
  > undefined
  >   `Undefined` or a subclass of it that is used to represent undefined values in the template.
  > finalize
  >   A callable that finalizes the variable. Per default no finalizing is applied.
  > autoescape
  >   If set to true the XML/HTML autoescaping feature is enabled. For more details about auto escaping see `Markup`.
  > loader
  >   The template loader for this environment.
  > cache_size
  >   The size of the cache. Per default this is `50` which means that if more than 50 templates are loaded the loader will clean out the least recently used template. If the cache size is set to `0` templates are recompiled all the time, if the cache size is `-1` the cache will not be cleaned.
  > auto_reload
  >   Some loaders load templates from locations where the template sources may change (ie: file system or database). If auto_reload is set to True (default) every time a template is requested the loader checks if the source changed and if yes, it will reload the template. For higher performance it’s possible to disable that.
  > bytecode_cache

  >   If set to a bytecode cache object, this object will provide a cache for the internal Jinja bytecode so that templates don’t have to be parsed if they were not changed.

  >   See Bytecode Cache for more information.

  `shared`
    If a template was created by using the `Template` constructor an environment is created automatically. These environments are created as shared environments which means that multiple templates may have the same anonymous environment. For all shared environments this attribute is True, else False.
  `sandboxed`
    If the environment is sandboxed this attribute is True. For the sandbox mode have a look at the documentation for the `SandboxedEnvironment`.
  `filters`
    A dict of filters for this environment. As long as no template was loaded it’s safe to add new filters or remove old. For custom filters see Custom Filters. For valid filter names have a look at Notes on Identifiers.
  `tests`
    A dict of test functions for this environment. As long as no template was loaded it’s safe to modify this dict. For custom tests see Custom Tests. For valid test names have a look at Notes on Identifiers.
  `globals`
    A dict of global variables. These variables are always available in a template. As long as no template was loaded it’s safe to modify this dict. For more details see The Global Namespace. For valid object names have a look at Notes on Identifiers.
  `overlay`([options])

    Create a new overlay environment that shares all the data with the current environment except of cache and the overriden attributes. Extensions cannot be removed for a overlayed environment. A overlayed environment automatically gets all the extensions of the environment it is linked to plus optional extra extensions.

    Creating overlays should happen after the initial environment was set up completely. Not all attributes are truly linked, some are just copied over so modifications on the original environment may not shine through.

  `undefined`([hint, obj, name, exc])

    Creates a new `Undefined` object for name. This is useful for filters or functions that may return undefined objects for some operations. All parameters except of hint should be provided as keyword parameters for better readability. The hint is used as error message for the exception if provided, otherwise the error message generated from obj and name automatically. The exception provided as exc is raised if something with the generated undefined object is done that the undefined object does not allow. The default exception is `UndefinedError`. If a hint is provided the name may be ommited.

    The most common way to create an undefined object is by providing a name only:

        return environment.undefined(name='some_name')

    This means that the name some_name is not defined. If the name was from an attribute of an object it makes sense to tell the undefined object the holder object to improve the error message:

        if not hasattr(obj, 'attr'):
            return environment.undefined(obj=obj, name='attr')

    For a more complex example you can provide a hint. For example the `first()` filter creates an undefined object that way:

        return environment.undefined('no first item, sequence was empty')

    If it the name or obj is known (for example because an attribute was accessed) it shold be passed to the undefined object, even if a custom hint is provided. This gives undefined objects the possibility to enhance the error message.

  `from_string`(source, globals=None, template_class=None)
    Load a template from a string. This parses the source given and returns a `Template` object.
  `get_template`(name, parent=None, globals=None)

    Load a template from the loader. If a loader is configured this method ask the loader for the template and returns a `Template`. If the parent parameter is not None, `join_path()` is called to get the real template name before loading.

    The globals parameter can be used to provide template wide globals. These variables are available in the context at render time.

    If the template does not exist a `TemplateNotFound` exception is raised.

  `join_path`(template, parent)

    Join a template with the parent. By default all the lookups are relative to the loader root so this method returns the template parameter unchanged, but if the paths should be relative to the parent template, this function can be used to calculate the real template name.

    Subclasses may override this method and implement template path joining here.

  `extend`(**attributes)
    Add the items to the instance of the environment if they do not exist yet. This is used by extensions to register callbacks and configuration values without breaking inheritance.
  `compile_expression`(source, undefined_to_none=True)

    A handy helper method that returns a callable that accepts keyword arguments that appear as variables in the expression. If called it returns the result of the expression.

    This is useful if applications want to use the same rules as Jinja in template “configuration files” or similar situations.

    Example usage:

        >>> env = Environment()
        >>> expr = env.compile_expression('foo == 42')
        >>> expr(foo=23)
        False
        >>> expr(foo=42)
        True

    Per default the return value is converted to None if the expression returns an undefined value. This can be changed by setting undefined_to_none to False.

        >>> env.compile_expression('var')() is None
        True
        >>> env.compile_expression('var', undefined_to_none=False)()
        Undefined

    new in Jinja 2.1

class `jinja2.` `Template`

  The central template object. This class represents a compiled template and is used to evaluate it.

  Normally the template object is generated from an `Environment` but it also has a constructor that makes it possible to create a template instance directly using the constructor. It takes the same arguments as the environment constructor but it’s not possible to specify a loader.

  Every template object has a few methods and members that are guaranteed to exist. However it’s important that a template object should be considered immutable. Modifications on the object are not supported.

  Template objects created from the constructor rather than an environment do have an environment attribute that points to a temporary environment that is probably shared with other templates created with the constructor and compatible settings.

      >>> template = Template('Hello {{ name }}!')
      >>> template.render(name='John Doe')
      u'Hello John Doe!'

      >>> stream = template.stream(name='John Doe')
      >>> stream.next()
      u'Hello John Doe!'
      >>> stream.next()
      Traceback (most recent call last):
          ...
      StopIteration

  `globals`
    The dict with the globals of that template. It’s unsafe to modify this dict as it may be shared with other templates or the environment that loaded the template.
  `name`
    The loading name of the template. If the template was loaded from a string this is None.
  `filename`
    The filename of the template on the file system if it was loaded from there. Otherwise this is None.
  `render`([context])

    This method accepts the same arguments as the dict constructor: A dict, a dict subclass or some keyword arguments. If no arguments are given the context will be empty. These two calls do the same:

        template.render(knights='that say nih')
        template.render({'knights': 'that say nih'})

    This will return the rendered template as unicode string.

  `generate`([context])

    For very large templates it can be useful to not render the whole template at once but evaluate each statement after another and yield piece for piece. This method basically does exactly that and returns a generator that yields one item after another as unicode strings.

    It accepts the same arguments as `render()`.

  `stream`([context])
    Works exactly like `generate()` but returns a `TemplateStream`.
  `module`

    The template as module. This is used for imports in the template runtime but is also useful if one wants to access exported template variables from the Python layer:

        >>> t = Template('{% macro foo() %}42{% endmacro %}23')
        >>> unicode(t.module)
        u'23'
        >>> t.module.foo()
        u'42'

  `make_module`(vars=None, shared=False, locals=None)
    This method works like the `module` attribute when called without arguments but it will evaluate the template every call rather then caching the template. It’s also possible to provide a dict which is then used as context. The arguments are the same as for the `new_context()` method.

class `jinja2.environment.` `TemplateStream`

  A template stream works pretty much like an ordinary python generator but it can buffer multiple items to reduce the number of total iterations. Per default the output is unbuffered which means that for every unbuffered instruction in the template one unicode string is yielded.

  If buffering is enabled with a buffer size of 5, five items are combined into a new unicode string. This is mainly useful if you are streaming big templates to a client via WSGI which flushes after each iteration.

  `disable_buffering`()
    Disable the output buffering.
  `enable_buffering`(size=5)
    Enable buffering. Buffer size items before yielding them.
  `dump`(fp, encoding=None, errors='strict')

    Dump the complete stream into a file or file-like object. Per default unicode strings are written, if you want to encode before writing specifiy an encoding.

    Example usage:

        Template('Hello {{ name }}!').stream(name='foo').dump('hello.html')

## Notes on Identifiers

Jinja2 uses the regular Python 2.x naming rules. Valid identifiers have to match `[a-zA-Z_][a-zA-Z0-9_]*`. As a matter of fact non ASCII characters are currently not allowed. This limitation will probably go away as soon as unicode identifiers are fully specified for Python 3.

Filters and tests are looked up in separate namespaces and have slightly modified identifier syntax. Filters and tests may contain dots to group filters and tests by topic. For example it’s perfectly valid to add a function into the filter dict and call it to.unicode. The regular expression for filter and test identifiers is `[a-zA-Z_][a-zA-Z0-9_]*(\.[a-zA-Z_][a-zA-Z0-9_]*)*``.

## Undefined Types

These classes can be used as undefined types. The `Environment` constructor takes an undefined parameter that can be one of those classes or a custom subclass of `Undefined`. Whenever the template engine is unable to look up a name or access an attribute one of those objects is created and returned. Some operations on undefined values are then allowed, others fail.

The closest to regular Python behavior is the StrictUndefined which disallows all operations beside testing if it’s an undefined object.

class `jinja2.` `Undefined`

  The default undefined type. This undefined type can be printed and iterated over, but every other access will raise an `UndefinedError`:

      >>> foo = Undefined(name='foo')
      >>> str(foo)
      ''
      >>> not foo
      True
      >>> foo + 42
      Traceback (most recent call last):
        ...
      UndefinedError: 'foo' is undefined

  `_undefined_hint`
    Either None or an unicode string with the error message for the undefined object.
  `_undefined_obj`
    Either None or the owner object that caused the undefined object to be created (for example because an attribute does not exist).
  `_undefined_name`
    The name for the undefined variable / attribute or just None if no such information exists.
  `_undefined_exception`
    The exception that the undefined object wants to raise. This is usually one of `UndefinedError` or `SecurityError`.
  `_fail_with_undefined_error`(*args, **kwargs)
    When called with any arguments this method raises `_undefined_exception` with an error message generated from the undefined hints stored on the undefined object.

class `jinja2.` `DebugUndefined`

  An undefined that returns the debug info when printed.

      >>> foo = DebugUndefined(name='foo')
      >>> str(foo)
      '{{ foo }}'
      >>> not foo
      True
      >>> foo + 42
      Traceback (most recent call last):
        ...
      UndefinedError: 'foo' is undefined

class `jinja2.` `StrictUndefined`

  An undefined that barks on print and iteration as well as boolean tests and all kinds of comparisons. In other words: you can do nothing with it except checking if it’s defined using the defined test.

      >>> foo = StrictUndefined(name='foo')
      >>> str(foo)
      Traceback (most recent call last):
        ...
      UndefinedError: 'foo' is undefined
      >>> not foo
      Traceback (most recent call last):
        ...
      UndefinedError: 'foo' is undefined
      >>> foo + 42
      Traceback (most recent call last):
        ...
      UndefinedError: 'foo' is undefined

Undefined objects are created by calling `undefined`.

Implementation

`Undefined` objects are implemented by overriding the special __underscore__ methods. For example the default `Undefined` class implements __unicode__ in a way that it returns an empty string, however __int__ and others still fail with an exception. To allow conversion to int by returning `0` you can implement your own:

    class NullUndefined(Undefined):
        def __int__(self):
            return 0
        def __float__(self):
            return 0.0

To disallow a method, just override it and raise `_undefined_exception`. Because this is a very common idom in undefined objects there is the helper method `_fail_with_undefined_error()` that does the error raising automatically. Here a class that works like the regular `Undefined` but chokes on iteration:

    class NonIterableUndefined(Undefined):
        __iter__ = Undefined._fail_with_undefined_error

## The Context

class `jinja2.runtime.` `Context`

  The template context holds the variables of a template. It stores the values passed to the template and also the names the template exports. Creating instances is neither supported nor useful as it’s created automatically at various stages of the template evaluation and should not be created by hand.

  The context is immutable. Modifications on `parent` must not happen and modifications on `vars` are allowed from generated template code only. Template filters and global functions marked as `contextfunction()`s get the active context passed as first argument and are allowed to access the context read-only.

  The template context supports read only dict operations (get, keys, values, items, iterkeys, itervalues, iteritems, __getitem__, __contains__). Additionally there is a `resolve()` method that doesn’t fail with a KeyError but returns an `Undefined` object for missing variables.

  `parent`
    A dict of read only, global variables the template looks up. These can either come from another `Context`, from the `Environment.globals` or `Template.globals` or points to a dict created by combining the globals with the variables passed to the render function. It must not be altered.
  `vars`
    The template local variables. This list contains environment and context functions from the `parent` scope as well as local modifications and exported variables from the template. The template will modify this dict during template evaluation but filters and context functions are not allowed to modify it.
  `environment`
    The environment that loaded the template.
  `exported_vars`
    This set contains all the names the template exports. The values for the names are in the `vars` dict. In order to get a copy of the exported variables as dict, `get_exported()` can be used.
  `name`
    The load name of the template owning this context.
  `blocks`
    A dict with the current mapping of blocks in the template. The keys in this dict are the names of the blocks, and the values a list of blocks registered. The last item in each list is the current active block (latest in the inheritance chain).
  `call`(callable, *args, **kwargs)
    Call the callable with the arguments and keyword arguments provided but inject the active context or environment as first argument if the callable is a `contextfunction()` or `environmentfunction()`.
  `resolve`(key)
    Looks up a variable like __getitem__ or get but returns an `Undefined` object with the name of the name looked up.
  `get_exported`()
    Get a new dict with the exported variables.
  `get_all`()
    Return a copy of the complete context as dict including the exported variables.

Implementation

Context is immutable for the same reason Python’s frame locals are immutable inside functions. Both Jinja2 and Python are not using the context / frame locals as data storage for variables but only as primary data source.

When a template accesses a variable the template does not define, Jinja2 looks up the variable in the context, after that the variable is treated as if it was defined in the template.

## Loaders

Loaders are responsible for loading templates from a resource such as the file system. The environment will keep the compiled modules in memory like Python’s sys.modules. Unlike sys.modules however this cache is limited in size by default and templates are automatically reloaded. All loaders are subclasses of `BaseLoader`. If you want to create your own loader, subclass `BaseLoader` and override get_source.

class `jinja2.` `BaseLoader`

  Baseclass for all loaders. Subclass this and override get_source to implement a custom loading mechanism. The environment provides a get_template method that calls the loader’s load method to get the `Template` object.

  A very basic example for a loader that looks up templates on the file system could look like this:

      from jinja2 import BaseLoader, TemplateNotFound
      from os.path import join, exists, getmtime

      class MyLoader(BaseLoader):

          def __init__(self, path):
              self.path = path

          def get_source(self, environment, template):
              path = join(self.path, template)
              if not exists(path):
                  raise TemplateNotFound(template)
              mtime = getmtime(path)
              with file(path) as f:
                  source = f.read().decode('utf-8')
              return source, path, lambda: mtime == getmtime(path)

  `get_source`(environment, template)

    Get the template source, filename and reload helper for a template. It’s passed the environment and template name and has to return a tuple in the form `(source, filename, uptodate)` or raise a TemplateNotFound error if it can’t locate the template.

    The source part of the returned tuple must be the source of the template as unicode string or a ASCII bytestring. The filename should be the name of the file on the filesystem if it was loaded from there, otherwise None. The filename is used by python for the tracebacks if no loader extension is used.

    The last item in the tuple is the uptodate function. If auto reloading is enabled it’s always called to check if the template changed. No arguments are passed so the function must store the old state somewhere (for example in a closure). If it returns False the template will be reloaded.

  `load`(environment, name, globals=None)
    Loads a template. This method looks up the template in the cache or loads one by calling `get_source()`. Subclasses should not override this method as loaders working on collections of other loaders (such as `PrefixLoader` or `ChoiceLoader`) will not call this method but get_source directly.

Here a list of the builtin loaders Jinja2 provides:

class `jinja2.` `FileSystemLoader`(searchpath, encoding='utf-8')

  Loads templates from the file system. This loader can find templates in folders on the file system and is the preferred way to load them.

  The loader takes the path to the templates as string, or if multiple locations are wanted a list of them which is then looked up in the given order:

      >>> loader = FileSystemLoader('/path/to/templates')
      >>> loader = FileSystemLoader(['/path/to/templates', '/other/path'])

  Per default the template encoding is `'utf-8'` which can be changed by setting the encoding parameter to something else.

class `jinja2.` `PackageLoader`(package_name, package_path='templates', encoding='utf-8')

  Load templates from python eggs or packages. It is constructed with the name of the python package and the path to the templates in that package:

      >>> loader = PackageLoader('mypackage', 'views')

  If the package path is not given, `'templates'` is assumed.

  Per default the template encoding is `'utf-8'` which can be changed by setting the encoding parameter to something else. Due to the nature of eggs it’s only possible to reload templates if the package was loaded from the file system and not a zip file.

class `jinja2.` `DictLoader`(mapping)

  Loads a template from a python dict. It’s passed a dict of unicode strings bound to template names. This loader is useful for unittesting:

      >>> loader = DictLoader({'index.html': 'source here'})

  Because auto reloading is rarely useful this is disabled per default.

class `jinja2.` `FunctionLoader`(load_func)

  A loader that is passed a function which does the loading. The function becomes the name of the template passed and has to return either an unicode string with the template source, a tuple in the form `(source, filename, uptodatefunc)` or None if the template does not exist.

      >>> def load_template(name):
      ...     if name == 'index.html'
      ...         return '...'
      ...
      >>> loader = FunctionLoader(load_template)

  The uptodatefunc is a function that is called if autoreload is enabled and has to return True if the template is still up to date. For more details have a look at `BaseLoader.get_source()` which has the same return value.

class `jinja2.` `PrefixLoader`(mapping, delimiter='/')

  A loader that is passed a dict of loaders where each loader is bound to a prefix. The prefix is delimited from the template by a slash per default, which can be changed by setting the delimiter argument to something else.

      >>> loader = PrefixLoader({
      ...     'app1':     PackageLoader('mypackage.app1'),
      ...     'app2':     PackageLoader('mypackage.app2')
      ... })

  By loading `'app1/index.html'` the file from the app1 package is loaded, by loading `'app2/index.html'` the file from the second.

class `jinja2.` `ChoiceLoader`(loaders)

  This loader works like the PrefixLoader just that no prefix is specified. If a template could not be found by one loader the next one is tried.

      >>> loader = ChoiceLoader([
      ...     FileSystemLoader('/path/to/user/templates'),
      ...     PackageLoader('mypackage')
      ... ])

  This is useful if you want to allow users to override builtin templates from a different location.

## Bytecode Cache

Jinja 2.1 and higher support external bytecode caching. Bytecode caches make it possible to store the generated bytecode on the file system or a different location to avoid parsing the templates on first use.

This is especially useful if you have a web application that is initialized on the first request and Jinja compiles many templates at once which slows down the application.

To use a bytecode cache, instanciate it and pass it to the `Environment`.

class `jinja2.` `BytecodeCache`

  To implement your own bytecode cache you have to subclass this class and override `load_bytecode()` and `dump_bytecode()`. Both of these methods are passed a `Bucket`.

  A very basic bytecode cache that saves the bytecode on the file system:

      from os import path

      class MyCache(BytecodeCache):

          def __init__(self, directory):
              self.directory = directory

          def load_bytecode(self, bucket):
              filename = path.join(self.directory, bucket.key)
              if path.exists(filename):
                  with file(filename, 'rb') as f:
                      bucket.load_bytecode(f)

          def dump_bytecode(self, bucket):
              filename = path.join(self.directory, bucket.key)
              with file(filename, 'wb') as f:
                  bucket.write_bytecode(f)

  A more advanced version of a filesystem based bytecode cache is part of Jinja2.

  `load_bytecode`(bucket)
    Subclasses have to override this method to load bytecode into a bucket. If they are not able to find code in the cache for the bucket, it must not do anything.
  `dump_bytecode`(bucket)
    Subclasses have to override this method to write the bytecode from a bucket back to the cache. If it unable to do so it must not fail silently but raise an exception.
  `clear`()
    Clears the cache. This method is not used by Jinja2 but should be implemented to allow applications to clear the bytecode cache used by a particular environment.

class `jinja2.bccache.` `Bucket`(environment, key, checksum)

  Buckets are used to store the bytecode for one template. It’s created and initialized by the bytecode cache and passed to the loading functions.

  The buckets get an internal checksum from the cache assigned and use this to automatically reject outdated cache material. Individual bytecode cache subclasses don’t have to care about cache invalidation.

  `environment`
    The `Environment` that created the bucket.
  `key`
    The unique cache key for this bucket
  `code`
    The bytecode if it’s loaded, otherwise None.
  `write_bytecode`(f)
    Dump the bytecode into the file or file like object passed.
  `load_bytecode`(f)
    Loads bytecode from a file or file like object.
  `bytecode_from_string`(string)
    Load bytecode from a string.
  `bytecode_to_string`()
    Return the bytecode as string.
  `reset`()
    Resets the bucket (unloads the bytecode).

Builtin bytecode caches:

class `jinja2.` `FileSystemBytecodeCache`(directory=None, pattern='__jinja2_%s.cache')

  A bytecode cache that stores bytecode on the filesystem. It accepts two arguments: The directory where the cache items are stored and a pattern string that is used to build the filename.

  If no directory is specified the system temporary items folder is used.

  The pattern can be used to have multiple separate caches operate on the same directory. The default pattern is `'__jinja2_%s.cache'`. `%s` is replaced with the cache key.

      >>> bcc = FileSystemBytecodeCache('/tmp/jinja_cache', '%s.cache')

  This bytecode cache supports clearing of the cache using the clear method.

class `jinja2.` `MemcachedBytecodeCache`(client, prefix='jinja2/bytecode/', timeout=None)

  This class implements a bytecode cache that uses a memcache cache for storing the information. It does not enforce a specific memcache library (tummy’s memcache or cmemcache) but will accept any class that provides the minimal interface required.

  Libraries compatible with this class:

  - werkzeug.contrib.cache
  - python-memcached
  - cmemcache

  (Unfortunately the django cache interface is not compatible because it does not support storing binary data, only unicode. You can however pass the underlying cache client to the bytecode cache which is available as django.core.cache.cache._client.)

  The minimal interface for the client passed to the constructor is this:

  class `MinimalClientInterface`
    `set`(key, value[, timeout])
      Stores the bytecode in the cache. value is a string and timeout the timeout of the key. If timeout is not provided a default timeout or no timeout should be assumed, if it’s provided it’s an integer with the number of seconds the cache item should exist.
    `get`(key)
      Returns the value for the cache key. If the item does not exist in the cache the return value must be None.

  The other arguments to the constructor are the prefix for all keys that is added before the actual cache key and the timeout for the bytecode in the cache system. We recommend a high (or no) timeout.

  This bytecode cache does not support clearing of used items in the cache. The clear method is a no-operation function.

## Utilities

These helper functions and classes are useful if you add custom filters or functions to a Jinja2 environment.

`jinja2.` `environmentfilter`(f)
  Decorator for marking evironment dependent filters. The current `Environment` is passed to the filter as first argument.

`jinja2.` `contextfilter`(f)
  Decorator for marking context dependent filters. The current `Context` will be passed as first argument.

`jinja2.` `environmentfunction`(f)
  This decorator can be used to mark a function or method as environment callable. This decorator works exactly like the `contextfunction()` decorator just that the first argument is the active `Environment` and not context.

`jinja2.` `contextfunction`(f)

  This decorator can be used to mark a function or method context callable. A context callable is passed the active `Context` as first argument when called from the template. This is useful if a function wants to get access to the context or functions provided on the context object. For example a function that returns a sorted list of template variables the current template exports could look like this:

      @contextfunction
      def get_exported_names(context):
          return sorted(context.exported_vars)

`jinja2.` `escape`(s)

  Convert the characters `&`, `<`, `>`, `'`, and `"` in string s to HTML-safe sequences. Use this if you need to display text that might contain such characters in HTML. This function will not escaped objects that do have an HTML representation such as already escaped data.

  The return value is a `Markup` string.

`jinja2.` `clear_caches`()
  Jinja2 keeps internal caches for environments and lexers. These are used so that Jinja2 doesn’t have to recreate environments and lexers all the time. Normally you don’t have to care about that but if you are messuring memory consumption you may want to clean the caches.

`jinja2.` `is_undefined`(obj)

  Check if the object passed is undefined. This does nothing more than performing an instance check against `Undefined` but looks nicer. This can be used for custom filters or tests that want to react to undefined variables. For example a custom default filter can look like this:

      def default(var, default=''):
          if is_undefined(var):
              return default
          return var

class `jinja2.` `Markup`([string])

  Marks a string as being safe for inclusion in HTML/XML output without needing to be escaped. This implements the __html__ interface a couple of frameworks and web applications use. `Markup` is a direct subclass of unicode and provides all the methods of unicode just that it escapes arguments passed and always returns Markup.

  The escape function returns markup objects so that double escaping can’t happen. If you want to use autoescaping in Jinja just enable the autoescaping feature in the environment.

  The constructor of the `Markup` class can be used for three different things: When passed an unicode object it’s assumed to be safe, when passed an object with an HTML representation (has an __html__ method) that representation is used, otherwise the object passed is converted into a unicode string and then assumed to be safe:

      >>> Markup("Hello <em>World</em>!")
      Markup(u'Hello <em>World</em>!')
      >>> class Foo(object):
      ...  def __html__(self):
      ...   return '<a href="#">foo</a>'
      ...
      >>> Markup(Foo())
      Markup(u'<a href="#">foo</a>')

  If you want object passed being always treated as unsafe you can use the `escape()` classmethod to create a `Markup` object:

      >>> Markup.escape("Hello <em>World</em>!")
      Markup(u'Hello &lt;em&gt;World&lt;/em&gt;!')

  Operations on a markup string are markup aware which means that all arguments are passed through the `escape()` function:

      >>> em = Markup("<em>%s</em>")
      >>> em % "foo & bar"
      Markup(u'<em>foo &amp; bar</em>')
      >>> strong = Markup("<strong>%(text)s</strong>")
      >>> strong % {'text': '<blink>hacker here</blink>'}
      Markup(u'<strong>&lt;blink&gt;hacker here&lt;/blink&gt;</strong>')
      >>> Markup("<em>Hello</em> ") + "<foo>"
      Markup(u'<em>Hello</em> &lt;foo&gt;')

  static `escape`(s)
    Escape the string. Works like `escape()` with the difference that for subclasses of `Markup` this function would return the correct subclass.
  `unescape`()

    Unescape markup again into an unicode string. This also resolves known HTML4 and XHTML entities:

        >>> Markup("Main &raquo; <em>About</em>").unescape()
        u'Main \xbb <em>About</em>'

  `striptags`()

    Unescape markup into an unicode string and strip all tags. This also resolves known HTML4 and XHTML entities. Whitespace is normalized to one:

        >>> Markup("Main &raquo;  <em>About</em>").striptags()
        u'Main \xbb About'

Note

The Jinja2 `Markup` class is compatible with at least Pylons and Genshi. It’s expected that more template engines and framework will pick up the __html__ concept soon.

## Exceptions

exception `jinja2.` `TemplateError`
  Baseclass for all template errors.

exception `jinja2.` `UndefinedError`
  Raised if a template tries to operate on `Undefined`.

exception `jinja2.` `TemplateNotFound`
  Raised if a template does not exist.

exception `jinja2.` `TemplateSyntaxError`

  Raised to tell the user that there is a problem with the template.

  `message`
    The error message as utf-8 bytestring.
  `lineno`
    The line number where the error occurred
  `name`
    The load name for the template as unicode string.
  `filename`
    The filename that loaded the template as bytestring in the encoding of the file system (most likely utf-8 or mbcs on Windows systems).

  The reason why the filename and error message are bytestrings and not unicode strings is that Python 2.x is not using unicode for exceptions and tracebacks as well as the compiler. This will change with Python 3.

exception `jinja2.` `TemplateAssertionError`
  Like a template syntax error, but covers cases where something in the template caused an error at compile time that wasn’t necessarily caused by a syntax error. However it’s a direct subclass of `TemplateSyntaxError` and has the same attributes.

## Custom Filters

Custom filters are just regular Python functions that take the left side of the filter as first argument and the the arguments passed to the filter as extra arguments or keyword arguments.

For example in the filter `{{ 42|myfilter(23) }}` the function would be called with `myfilter(42, 23)`. Here for example a simple filter that can be applied to datetime objects to format them:

    def datetimeformat(value, format='%H:%M / %d-%m-%Y'):
        return value.strftime(format)

You can register it on the template environment by updating the `filters` dict on the environment:

    environment.filters['datetimeformat'] = datetimeformat

Inside the template it can then be used as follows:

    written on: {{ article.pub_date|datetimeformat }}
    publication date: {{ article.pub_date|datetimeformat('%d-%m-%Y') }}

Filters can also be passed the current template context or environment. This is useful if a filters wants to return an undefined value or check the current `autoescape` setting. For this purpose two decorators exist: `environmentfilter()` and `contextfilter()`.

Here a small example filter that breaks a text into HTML line breaks and paragraphs and marks the return value as safe HTML string if autoescaping is enabled:

    import re
    from jinja2 import environmentfilter, Markup, escape

    _paragraph_re = re.compile(r'(?:\r\n|\r|\n){2,}')

    @environmentfilter
    def nl2br(environment, value):
        result = u'\n\n'.join(u'<p>%s</p>' % p.replace('\n', '<br>\n')
                              for p in _paragraph_re.split(escape(value)))
        if environment.autoescape:
            result = Markup(result)
        return result

Context filters work the same just that the first argument is the current active `Context` rather then the environment.

## Custom Tests

Tests work like filters just that there is no way for a test to get access to the environment or context and that they can’t be chained. The return value of a test should be True or False. The purpose of a test is to give the template designers the possibility to perform type and conformability checks.

Here a simple test that checks if a variable is a prime number:

    import math

    def is_prime(n):
        if n == 2:
            return True
        for i in xrange(2, int(math.ceil(math.sqrt(n))) + 1):
            if n % i == 0:
                return False
        return True

You can register it on the template environment by updating the `tests` dict on the environment:

    environment.tests['prime'] = is_prime

A template designer can then use the test like this:

    {% if 42 is prime %}
        42 is a prime number
    {% else %}
        42 is not a prime number
    {% endif %}

## The Global Namespace

Variables stored in the `Environment.globals` dict are special as they are available for imported templates too, even if they are imported without context. This is the place where you can put variables and functions that should be available all the time. Additionally `Template.globals` exist that are variables available to a specific template that are available to all `render()` calls.

## Low Level API

The low level API exposes functionality that can be useful to understand some implementation details, debugging purposes or advanced extension techniques. Unless you know exactly what you are doing we don’t recommend using any of those.

`Environment.` `lex`(source, name=None, filename=None)

  Lex the given sourcecode and return a generator that yields tokens as tuples in the form `(lineno, token_type, value)`. This can be useful for extension development and debugging templates.

  This does not perform preprocessing. If you want the preprocessing of the extensions to be applied you have to filter source through the `preprocess()` method.

`Environment.` `parse`(source, name=None, filename=None)

  Parse the sourcecode and return the abstract syntax tree. This tree of nodes is used by the compiler to convert the template into executable source- or bytecode. This is useful for debugging or to extract information from templates.

  If you are developing Jinja2 extensions this gives you a good overview of the node tree generated.

`Environment.` `preprocess`(source, name=None, filename=None)
  Preprocesses the source with all extensions. This is automatically called for all parsing and compiling methods but not for `lex()` because there you usually only want the actual source tokenized.

`Template.` `new_context`(vars=None, shared=False, locals=None)

  Create a new `Context` for this template. The vars provided will be passed to the template. Per default the globals are added to the context. If shared is set to True the data is passed as it to the context without adding the globals.

  locals can be a dict of local variables for internal usage.

`Template.` `root_render_func`(context)

  This is the low level render function. It’s passed a `Context` that has to be created by `new_context()` of the same template or a compatible template. This render function is generated by the compiler from the template code and returns a generator that yields unicode strings.

  If an exception in the template code happens the template engine will not rewrite the exception but pass through the original one. As a matter of fact this function should only be called from within a `render()` / `generate()` / `stream()` call.

`Template.` `blocks`
  A dict of block render functions. Each of these functions works exactly like the `root_render_func()` with the same limitations.

`Template.` `is_up_to_date`
  This attribute is False if there is a newer version of the template available, otherwise True.

Note

The low-level API is fragile. Future Jinja2 versions will try not to change it in a backwards incompatible way but modifications in the Jinja2 core may shine through. For example if Jinja2 introduces a new AST node in later versions that may be returned by `parse()`.